// -*- C++ -*-
/*!
 * @file  GpsNavigationComponent.cpp
 * @brief GpsNavigationComponent
 * @date $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川（HRI）研究室
 * 代表者：水川　真 (Email:mizukawa<at>sic.shibaura-it.ac.jp)
 *
 * GpsNavigationComponentについて著作権は，芝浦工業大学水川研究室に帰属します．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#include "GpsNavigationComponent.h"

// Module specification
// <rtc-template block="module_spec">
static const char* gpsnavigationcomponent_spec[] =
  {
    "implementation_id", "GpsNavigationComponent",
    "type_name",         "GpsNavigationComponent",
    "description",       "GpsNavigationComponent",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "Navigation",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.str_mapname", "map.txt",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
GpsNavigationComponent::GpsNavigationComponent(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Data_inIn("Data_in", m_Data_in),
    m_Distance_outOut("Distance_out", m_Distance_out),
    m_Degree_outOut("Degree_out", m_Degree_out),
    m_MapData_outOut("MapData_out", m_MapData_out)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
GpsNavigationComponent::~GpsNavigationComponent()
{
}



RTC::ReturnCode_t GpsNavigationComponent::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Data_in", m_Data_inIn);
  
  // Set OutPort buffer
  registerOutPort("Distance_out", m_Distance_outOut);
  registerOutPort("Degree_out", m_Degree_outOut);
  registerOutPort("MapData_out", m_MapData_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("str_mapname", m_str_mapname, "map.txt");
  
  std::cout<< "/--GPS_Navigation_Component--/" <<std::endl;	

  std::cout<< "/--Please set configuration of mapdata--/" <<std::endl;


  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GpsNavigationComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsNavigationComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsNavigationComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * 指定されたマップの確認を行う
 */

RTC::ReturnCode_t GpsNavigationComponent::onActivated(RTC::UniqueId ec_id)
{
	m_configsets.update();

	std::cout<< "Map load:: " <<  m_str_mapname << " start" <<std::endl;
	if(gps.set_mapArray( m_str_mapname ) ){
		std::cout<< "OK....Map load end" << std::endl;
	}

	//マップを開けなかった場合はerrorを表示
	else std::cout<<"erorr!!"<<std::endl;

	return RTC::RTC_OK;
}


RTC::ReturnCode_t GpsNavigationComponent::onDeactivated(RTC::UniqueId ec_id)
{
	gps.initialize();
	return RTC::RTC_OK;
}

/*!
 * 現在の位置データと前回の位置データからロボットの姿勢を計算し，次に向かう点まで
 * の距離や角度の計算を行う．
 */

RTC::ReturnCode_t GpsNavigationComponent::onExecute(RTC::UniqueId ec_id)
{
	//入力ポートからデータを取得
	if(m_Data_inIn.isNew() && gps.get_state() ){
		m_Data_inIn.read();
		for(unsigned int i=0;i<m_Data_in.data.length();i++){
			tem=m_Data_in.data[i];
			str_tem.append( &tem );
			gpsdata.append( str_tem.substr( 0 , 1 ) );
			str_tem.erase( 0 , str_tem.length() );	
		}
		if( gps.update_navigation( gpsdata.c_str() ) ){
			//*************************outポートへ出力*****************************************
			m_Distance_out.data=gps.get_distance();		
			m_Degree_out.data=gps.get_degree();
			m_MapData_out.data=(short)gps.get_map_count();

			m_Distance_outOut.write();
			m_Degree_outOut.write();
			m_MapData_outOut.write();		
			//*********************************************************************************
			std::cout << gps.get_distance() << "::" << gps.get_degree() << "::" << gps.get_map_count() <<std::endl;	
		}
		gpsdata.erase( 0 , gpsdata.length() );
	}
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GpsNavigationComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsNavigationComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsNavigationComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsNavigationComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsNavigationComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void GpsNavigationComponentInit(RTC::Manager* manager)
  {
    coil::Properties profile(gpsnavigationcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<GpsNavigationComponent>,
                             RTC::Delete<GpsNavigationComponent>);
  }
  
};


