// -*- C++ -*-
/*!
 * @file  GpsReceiverComponent.cpp
 * @brief GpsReceiverComponent
 * @date $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟 11-Q-32
 * 水川(HRI)研究室
 * 代表者：水川　真 (Email:mizukawa<at>sic.shibaura-it.ac.jp)
 *
 * GpsReceiverComponentについての著作権は，芝浦工業大学水川研究室に帰属します．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#include "GpsReceiverComponent.h"

// Module specification
// <rtc-template block="module_spec">
static const char* gpsreceivercomponent_spec[] =
  {
    "implementation_id", "GpsReceiverComponent",
    "type_name",         "GpsReceiverComponent",
    "description",       "GpsReceiverComponent",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "GPS Receiver",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.str_portname", "COM1",
    "conf.default.int_baudrate", "19200",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
GpsReceiverComponent::GpsReceiverComponent(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_OutDataOut("OutData", m_OutData)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
GpsReceiverComponent::~GpsReceiverComponent()
{
}



RTC::ReturnCode_t GpsReceiverComponent::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("OutData", m_OutDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("str_portname", m_str_portname, "COM1");
  bindParameter("int_baudrate", m_int_baudrate, "19200");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GpsReceiverComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsReceiverComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsReceiverComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * GPSのシリアルポートの設定を行う
 */

RTC::ReturnCode_t GpsReceiverComponent::onActivated(RTC::UniqueId ec_id)
{
	m_configsets.update();

	//シリアルポートを開く
	hComm = CreateFile( 
		m_str_portname.c_str(),				//COM１の開放
		GENERIC_READ | GENERIC_WRITE,	//読み書きの許可
		0,								//共有モードの設定
		NULL,							//取得したハンドルを子プロセスへ継承することを許可するかどうか
		OPEN_EXISTING,					//ファイルの扱い方の指定，最初から存在しているポートの関してはOPEN_EXISTINGを指定
		FILE_ATTRIBUTE_NORMAL,			//このファイルに特に属性を設定しません．非同期で通信する場合はFILE_FLAG_OVERLAPPED
		NULL							//シリアル通信ではNULLを指定
		);

	if( hComm == INVALID_HANDLE_VALUE ){
		std::cout<<"ポートを開くことができません"<<std::endl;
		return RTC::RTC_ERROR;
	}

	//シリアルポートの設定
	DCB dbc;								//DBC構造体の変数dbcを宣言．以下シリアルポートの設定はこの構造体に設定
	GetCommState(hComm, &dbc);				//DBCの取得
	dbc.BaudRate=m_int_baudrate;			//通信速度の設定
	dbc.ByteSize=8;							//データのサイズの設定
	dbc.Parity=NOPARITY;					//パリティの設定(使用しない，他にはEVENPARITY,ODDPARITY)
	dbc.fParity=FALSE;						//パリティを使用するか
	dbc.StopBits=ONESTOPBIT;				//ストップビットの設定
	dbc.fOutxCtsFlow=FALSE;					//送信時に，CTSを監視するかどうか
	dbc.fOutxDsrFlow=FALSE;					//送信時に，DSRを監視するかどうか
	dbc.fDsrSensitivity=FALSE;				//DSRがOFFの間は受信データを無視するか
	SetCommState(hComm, &dbc);				//DBCを設定
	
	return RTC::RTC_OK;
}

/*!
 * GPSのシリアルポートを閉じる
 */

RTC::ReturnCode_t GpsReceiverComponent::onDeactivated(RTC::UniqueId ec_id)
{
	//COMを閉じる
	CloseHandle(hComm);

	return RTC::RTC_OK;
}

/*!
 * GPSのデータを受信し，アウトポートから出力する
 */

RTC::ReturnCode_t GpsReceiverComponent::onExecute(RTC::UniqueId ec_id)
{
	char rdBuf[255];
	unsigned int i=0;
	//****************************受信用(ReadFile)******************************************
	//エラー回避のためのバイト数チェック
	ClearCommError(hComm,&dwErrors,&ComStat);
	dwCount=ComStat.cbInQue;
	if(dwCount!=0){
		if(dwCount>=256) dwCount=255;
		ZeroMemory(rdBuf, 256);
		//受信コマンド
		ReadFile(hComm,rdBuf,dwCount,&dwRead,NULL);
		std::cout<<rdBuf;
		m_OutData.data.length(strlen(rdBuf));
		while( i<strlen(rdBuf) ){
			m_OutData.data[i] = rdBuf[i];
			i++;
		}
		while( i<m_OutData.data.length() ){
			m_OutData.data[i] = ' ';
			i++;
		}

		m_OutDataOut.write();
	}

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GpsReceiverComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsReceiverComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsReceiverComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsReceiverComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GpsReceiverComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void GpsReceiverComponentInit(RTC::Manager* manager)
  {
    coil::Properties profile(gpsreceivercomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<GpsReceiverComponent>,
                             RTC::Delete<GpsReceiverComponent>);
  }
  
};


